from django.contrib import admin
from django.utils.html import mark_safe

from django_deep_link.models import App, Visit


@admin.register(App)
class AppAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "android_app",
        "ios_app",
        "mac_app",
        "windows_app",
        "deep_link",
        "app_store_url",
        "play_store_url",
    )
    list_display_links = (
        "name",
        # "android_package_name",
        # "ios_bundle_id",
    )
    search_fields = (
        "name",
        # "android_package_name",
        # "ios_bundle_id",
    )
    readonly_fields = ("code",)

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "code",
                    "name",
                    # "default_url",
                ),
            },
        ),
        (
            "iOS Mobile",
            {
                # "classes": ("collapse",),
                "fields": (
                    "ios_url",
                    "ios_app",
                    "ios_uri_scheme",
                    "ios_bundle_id",
                    "ios_custom_url",
                ),
            },
        ),
        (
            "Android Mobile",
            {
                # "classes": ("collapse",),
                "fields": (
                    "android_url",
                    "android_app",
                    "android_uri_scheme",
                    "android_package_name",
                    "android_custom_url",
                ),
            },
        ),
        (
            "Mac Desktop",
            {
                # "classes": ("collapse",),
                "fields": (
                    "mac_app",
                    "mac_uri_scheme",
                    "mac_app_store_url",
                ),
            },
        ),
        (
            "Windows Desktop",
            {
                # "classes": ("collapse",),
                "fields": (
                    "windows_app",
                    "windows_uri_scheme",
                    "windows_app_store_url",
                    "windows_package_name",
                ),
            },
        ),
    )

    def deep_link(self, obj):
        return mark_safe(
            f'<a href="{obj.get_absolute_url()}" target="_blank">Visit</a>'
        )

    def app_store_url(self, obj):
        if obj.get_app_store_url:
            return mark_safe(f'<a href="{obj.get_app_store_url()}" target="_blank">Visit</a>')

    def play_store_url(self, obj):
        if obj.get_play_store_url:
            return mark_safe(
                f'<a href="{obj.get_play_store_url()}" target="_blank">Visit</a>'
            )

    class Media:
        js = ("django_deep_link/js/app.js",)


@admin.register(Visit)
class VisitAdmin(admin.ModelAdmin):
    list_display = (
        "ip_address",
        "deep_link",
        "browser",
        "os",
        "device",
        "platform",
        "created",
    )
    readonly_fields = (
        "ip_address",
        "ip_data",
        "ua_data",
        "query_data",
        "deep_link",
    )
    date_hierarchy = "created"
    list_filter = ("deep_link__name",)

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    class Media:
        css = {
            "all": (
                "//cdnjs.cloudflare.com/ajax/libs/highlight.js/11.1.0/styles/default.min.css",
            ),
        }
        js = (
            "//cdnjs.cloudflare.com/ajax/libs/highlight.js/11.1.0/highlight.min.js",
            # "django_deep_link/js/ace-builds/ace.js",
            "django_deep_link/js/scan.js",
        )



import logging

import requests
from django.conf import settings
from django.core.cache import cache

logger = logging.getLogger(__name__)


def get_ip_geodata(ip_address):
    """Use 3rd party API to obtain Geolocation data from a given IP."""

    if ip_address in settings.INTERNAL_IPS:
        logger.debug(
            "Not looking up IP gelocation data, IP is is found in INTERNAL_IPS"
        )
        return {}

    if cache.get(ip_address):
        return cache.get(ip_address)

    if not hasattr(settings, 'IPSTACK_ACCESS_KEY'):
        logger.warn(
            "Unable to lookup IP geolocation data, IPSTACK_ACCESS_KEY not in settings"
        )
        return {}

    params = {
        "fields": "main",
        "hostname": 1,
        "access_key": settings.IPSTACK_ACCESS_KEY,
    }
    r = requests.get(f"https://api.ipstack.com/{ip_address}", params=params)
    r.raise_for_status()
    cache.set(ip_address, r.json(), settings.IPSTACK_CACHE_TIME)
    return r.json()



from django.views.generic import DetailView
from ipware import get_client_ip
from user_agents import parse

from .helpers import get_querystring_as_dict
from .helpers.ua import get_platform_bools, ua_to_dict
from .models import App, Visit
from .settings import api_settings


class AppDownloadView(DetailView):
    """
    Shows an app download page if app not installed or redirects to the
    installed app if it is.
    """

    model = App
    template_name = "django_deep_link/app.html"

    def get_object(self, queryset=None):
        return App.objects.get(code=self.kwargs.get("code"))

    def get_context_data(self, **kwargs):
        # Get user agent data
        ua_string = self.request.META.get("HTTP_USER_AGENT", None)
        user_agent = parse(ua_string)
        ua_data = ua_to_dict(user_agent) if user_agent else {}

        # Get ip address data
        ip_address, _ = get_client_ip(self.request)
        if ip_address:
            get_ip_address_information = api_settings.IP_GEO_HANDLER
            ip_data = get_ip_address_information(ip_address)
            Visit.objects.create(
                ip_address=ip_address,
                ua_data=ua_data,
                ip_data=ip_data,
                query_data=get_querystring_as_dict(self.request),
                deep_link=self.object,
            )

        ctx = super().get_context_data(**kwargs)
        ctx["user_agent"] = user_agent
        ctx.update(get_platform_bools(user_agent))
        return ctx



def get_ua_platform(user_agent):
    """Get platform (mobile, tablet, pc)."""
    if user_agent.is_mobile:
        return "mobile"
    elif user_agent.is_tablet:
        return "tablet"
    elif user_agent.is_pc:
        return "pc"
    else:
        return "unknown"


def get_platform_bools(user_agent):
    """Return if we're on a pc, mobile or tablet platform."""

    is_pc_mac = False
    is_pc_windows = False
    is_mobile_ios = False
    is_mobile_android = False
    is_tablet_ios = False
    is_tablet_android = False

    if user_agent.is_mobile:
        if user_agent.os.family == "iOS":
            is_mobile_ios = True
        elif user_agent.os.family == "Android":
            is_mobile_android = True

    elif user_agent.is_tablet:
        if user_agent.os.family == "iOS":
            is_tablet_ios = True
        elif user_agent.os.family == "Android":
            is_tablet_android = True

    elif user_agent.is_pc:
        if user_agent.os.family == "Mac OS X":
            is_pc_mac = True
        elif user_agent.os.family == "Windows":
            is_pc_windows = True

    return {
        "is_pc_mac": is_pc_mac,
        "is_pc_windows": is_pc_windows,
        "is_mobile_ios": is_mobile_ios,
        "is_mobile_android": is_mobile_android,
        "is_tablet_ios": is_tablet_ios,
        "is_tablet_android": is_tablet_android,
    }


def ua_to_dict(user_agent):
    """Return User Agent data as a dict."""
    return {
        "browser": dict(user_agent.browser._asdict()),
        "os": dict(user_agent.os._asdict()),
        "device": dict(user_agent.device._asdict()),
        "platform": get_ua_platform(user_agent),
    }



# Generated by Django 3.2.5 on 2021-08-04 15:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_deep_link', '0006_auto_20210729_1527'),
    ]

    operations = [
        migrations.AlterField(
            model_name='visit',
            name='ip_data',
            field=models.JSONField(blank=True, default=dict, verbose_name='IP geodata'),
        ),
        migrations.AlterField(
            model_name='visit',
            name='query_data',
            field=models.JSONField(blank=True, default=dict, verbose_name='Query data'),
        ),
        migrations.AlterField(
            model_name='visit',
            name='ua_data',
            field=models.JSONField(blank=True, default=dict, verbose_name='User agent data'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-29 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_deep_link', '0004_auto_20210728_1722'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='visit',
            options={'default_permissions': ('delete', 'view')},
        ),
        migrations.AlterField(
            model_name='app',
            name='android_url',
            field=models.URLField(blank=True, help_text='Custom app download/coming soon page. If blank, users will not be redirected.', verbose_name='Android URL'),
        ),
        migrations.AlterField(
            model_name='app',
            name='default_url',
            field=models.URLField(help_text='Your fallback URL for platforms that do not have a specified redirect.', verbose_name='Default URL'),
        ),
        migrations.AlterField(
            model_name='app',
            name='ios_url',
            field=models.URLField(blank=True, help_text='Custom app download/coming soon page. If blank, users will not be redirected.', verbose_name='iOS URL'),
        ),
        migrations.AlterField(
            model_name='app',
            name='mac_app_store_url',
            field=models.URLField(blank=True, help_text='A URL to fallback to when the app is not installed.', max_length=255, verbose_name='Mac App Store URL'),
        ),
        migrations.AlterField(
            model_name='app',
            name='windows_app_store_url',
            field=models.URLField(blank=True, help_text='A URL to fallback to when the app is not installed.', max_length=255, verbose_name='Windows App Store URL'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-28 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_deep_link', '0002_auto_20210728_1540'),
    ]

    operations = [
        migrations.AlterField(
            model_name='app',
            name='mac_app',
            field=models.BooleanField(default=False, verbose_name='I have a Mac App'),
        ),
        migrations.AlterField(
            model_name='app',
            name='windows_app',
            field=models.BooleanField(default=False, verbose_name='I have a Windows App'),
        ),
    ]
